// -*- C++ -*-
#ifndef CREATEDOM_H
#define CREATEDOM_H

#include <string>
#include <vector>
#include "ComponentInfoContainer.h"

using namespace xercesc;

//namespace DAQMW {
class CreateDom
{
	struct Result {
		bool status;
		std::string code;
		std::string className;
		std::string name;
		std::string methodName;
		std::string messageEng;
		std::string messageJpn;
	};

public:
	CreateDom();
	~CreateDom();

	std::string getOK(std::string command);
	std::string getNG(std::string command, int code, 
			  std::string methodName, std::string messageEng, 
			  std::string messageJpn);
	std::string getParams(std::string command, NVList* list);
        std::string getStatus(std::string command, DAQLifeCycleState state);
        std::string getLog(std::string command, groupStatusList status_list);
        std::string getLog(std::string command, groupStatusList status_list,
			   std::string err_msg);
	std::string getCompName(ComponentName name);
	std::string getState(DAQLifeCycleState state, bool flag);
	
private:
	void makeRoot();
	void makeMethodnName(std::string name);
	void makeReturnvalue();
	void makeResultOK();
	void makeResultNG(Result result);
	void makeResult(Result result);
	void makeDevStatus();
	void make(DOMElement* ele, std::string tag, std::string text);
	void makeState(DAQLifeCycleState state);
	void makeGateBoard(bool open);
	void makeParams();
	void makeSitcp(int cnt);
	void makeIp(int cnt, std::string value);
	void makeName(int cnt, std::string value);
	void makeLevel(int cnt, int index, std::string value);
	void makeCo(int cnt, int index, std::string value);
	void makeDead(int cnt, int index, std::string value);
	void makeValue(DOMElement* ele, int index, std::string value);
	void makeLogs();
	void makeLog(groupStatus groupStatus);

	bool check(std::string name, int *cnt, int *type, int *index);
	int checkDigit(std::string name, int index, int cnt);

	std::string getBuffer();

private:
	std::string m_str_sitcp;
	std::string m_str_ip;
	std::string m_str_name;
	std::string m_str_level;
	std::string m_str_co;
	std::string m_str_dead;

	int m_cnt;
	
	DOMImplementation* m_impl;
	DOMDocument* m_doc;
	DOMElement* m_rootElem;
	DOMElement* m_returnElem;
	DOMElement* m_devStatusElem;
	DOMElement* m_paramsElem;
	DOMElement* m_sitcpElem;
	DOMElement* m_levelElem;
	DOMElement* m_coElem;
	DOMElement* m_deadElem;
	DOMElement* m_logsElem;
	DOMElement* m_logElem;
	
	bool  m_debug;
};

//}//namespace
#endif // CREATEDOM_H
