#include "EventDataToHistogramEmulator.h"

EventDataToHistogramEmulator::EventDataToHistogramEmulator(
	string MaskFileName,
	Double tof_min,
	Double tof_max,
	UInt4 num_of_tof_bin,
	UInt4 num_of_psd_per_module,
	UInt4 num_of_pixel_per_psd,
	UInt4 total_module_num
) {
	SetParam(
		MaskFileName,
		tof_min,
		tof_max,
		num_of_tof_bin,
		num_of_psd_per_module,
		num_of_pixel_per_psd,
		total_module_num
	);

}

//////////////////////////////////////////////////////////////////////

EventDataToHistogramEmulator::
~EventDataToHistogramEmulator(){
	std::cerr << "EventDataToHistogramEmulator Destructor" << std::endl;
	delete[] m_tof_bin;
}

//////////////////////////////////////////////////////////////////////

void EventDataToHistogramEmulator::
SetParam(
	string MaskFileName,
	Double tof_min,
	Double tof_max,
	UInt4 num_of_tof_bin,
	UInt4 num_of_psd_per_module,
	UInt4 num_of_pixel_per_psd,
	UInt4 total_module_num
) {

	m_num_of_psd_per_module = num_of_psd_per_module; // Usually 8
	m_num_of_pixel_per_psd  = num_of_pixel_per_psd;

	// DAY 2
	UInt4 num_of_psd = num_of_psd_per_module * total_module_num;

	Allocate( num_of_psd, num_of_psd * num_of_pixel_per_psd );
  
	// set PSD parameters for each PSD
	vector<Double> A(NumOfPsd);
	vector<Double> B(NumOfPsd);
	vector<Double> C(NumOfPsd);
	for( UInt4 i=0; i<NumOfPsd; i++ ){
		A[i] = (Double) (num_of_pixel_per_psd - 1.0);
		//A[i] = 511.0;
		B[i] = 1.0;
		C[i] = 0.0;
	}

	for( UInt4 i=0; i<NumOfPsd; i++ ){
		SetPsdParam( i, A[i], B[i], C[i] );
	}
  
	UInt4 Size = num_of_tof_bin;
	m_tof_bin = new Double [ Size ];

	for( UInt4 i=0; i<Size; i++ ){
	// RC2
    m_tof_bin[i] = (Double) tof_min +
		(Double)( ( (Double) tof_max - (Double) tof_min ) / num_of_tof_bin * i);
	}
	std::cerr << "m_tof_bin[0]"      << m_tof_bin[0]      << std::endl;
	std::cerr << "m_tof_bin[Size-1]" << m_tof_bin[Size-1] << std::endl;

	for( UInt4 i=0; i<NumOfPixel; i++ ){
		SetTofBin( i, m_tof_bin, Size );
	}
}

//////////////////////////////////////////////////////////////////////

UInt4 EventDataToHistogramEmulator::
PutPsdId(UInt4 DaqChannel, UInt4 module, UInt4 psd) {
//  //return DaqChannel + NUM_OF_PSD_PER_MODULE * module + psd;
//  //return DaqChannel + NUM_OF_PSD_PER_MODULE * 
//  return DaqChannel + m_num_of_psd_per_module * 
//		m_convert_module_num[m_crate_num][module] + psd;
	return 0; // DUMMY
}

//////////////////////////////////////////////////////////////////////

UInt4 EventDataToHistogramEmulator::
PutPsdId(UInt4 DaqChannel, UInt4 module, UInt4 psd, UInt4 modNo) {

	return DaqChannel + m_num_of_psd_per_module * modNo + psd;

}

//////////////////////////////////////////////////////////////////////

UInt4 EventDataToHistogramEmulator::
PutPixelId(UInt4 PsdId, UInt4 PixelPosition) {
	//return PsdId * NUM_OF_PIXEL_PER_PSD + PixelPosition;

	return PsdId * m_num_of_pixel_per_psd + PixelPosition;

}

//////////////////////////////////////////////////////////////////////
void EventDataToHistogramEmulator::
Increment(UInt4 Channel, UChar* data, UInt4 size, UInt4 modNo) {

	UChar *CopyData = new UChar [ size*8 ];
	for (UInt4 i = 0; i < size*8; i++ ){
		CopyData[i] = data[i];
	}

	UInt4 index = 0;
	for (UInt4 i=0; i<size; i++) {
		index = i * 8; // XXX: 8: event_byte_size
		if(*(data+index) == 0x5a) { // event data
      		Decode_EventData(
				CopyData+index,
				&_ModuleId,
				&_psd_num,
				&_k,
				&_ph_l,
				&_ph_r,
				&_tof
			);

			UInt4 psd_id   = PutPsdId(Channel, _ModuleId, _psd_num, modNo);
      		UInt4 pixel_id = PutPixelId(
								psd_id,
								PutPixelPosition( psd_id, _ph_l, _ph_r)
				 	   		 );
			((*gslhist)[pixel_id])->Increment(ClockToMicrosec(_tof));
    	}
    	else if(*(data+index) == 0x5b) { // T0 data
      		Decode_T0Data( CopyData+index, &_CrateId, &_PulseId );
		}
		else if(*(data+index) == 0x5c) { // time stamp data
			Decode_TimeStampData(
				CopyData+index,
				&_t1, &_t2, &_t3,
				&_t4, &_t5, &_t6
			);
    	}
    	else if(*(data+index) == 0x5d) { // 
			;
		}
	}
	delete [] CopyData;
}
//////////////////////////////////////////////////////////////////////
void EventDataToHistogramEmulator::
Decode_EventData(UChar *data, UInt4 *module_num, UInt4 *psd_num, UInt4 *k, 
		  UInt4 *ph_l, UInt4 *ph_r, UInt4 *tof) {

	*psd_num    = (UInt4)( *(data+4) & 0x7 );
	*module_num = (UInt4)( (*(data+4) & 0xf8) >> 3);
	*k          = (UInt4)( *(data+6) )/16;
	*ph_l       = (UInt4)( *(data+5) ) * 16 + ( *k );
	*ph_r       = (UInt4)( *(data+7) ) + 256 * ( (UInt4( *(data+6) ) )-(*k)*16 );
	*tof        = (UInt4)( *(data+1) << 16 ) +
			      (UInt4)( *(data+2) <<  8 ) +
			      (UInt4)( *(data+3)       ) ;
}
//////////////////////////////////////////////////////////////////////
void EventDataToHistogramEmulator::
Decode_T0Data(UChar *data, UInt4 *CrateId, UInt8 *PulseId) {
  *CrateId = (UInt4)(*(data+1));
  *PulseId = (UInt8) ((*(data+3)) * 256 * 256 * 256 * 256 +
					  (*(data+4)) * 256 * 256 * 256 +
					  (*(data+5)) * 256 * 256 +
					  (*(data+6)) * 256 +
					  (*(data+7)));
}
