// -*- C++ -*-
/*!
 * @file Gatenet.h
 * @brief Event data dispatching component. 
 * @date
 * @author Kazuo Nakayoshi <kazuo.nakayoshi@kek.jp>
 *
 * Copyright (C) 2008
 *     Kazuo Nakayoshi
 *     Electronics System Group,
 *     KEK, Japan.
 *     All rights reserved.
 *
 */

#ifndef GATENET_H
#define GATENET_H

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <rtm/idl/BasicDataTypeSkel.h>
#include "MlfComponent.h"
#include "MyRingBuffer.h"
#include "Condition.h"

// Service implementation headers
#include "DAQServiceSVC_impl.h"

using namespace RTC;

class Gatenet
    : public DAQMW::MlfComponent
{
public:
    Gatenet(RTC::Manager* manager);
    ~Gatenet();

    // The initialize action (on CREATED->ALIVE transition)
    // formaer rtc_init_entry() 
    virtual RTC::ReturnCode_t onInitialize();

    // The execution action that is invoked periodically
    // former rtc_active_do()
    virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

private:
    int daq_dummy();
    int daq_configure();
    int daq_unconfigure();
    int daq_start();
    int daq_run();
    int daq_stop();
    int daq_pause();
    int daq_resume();

    int parse_params(::NVList* list);

    int get_condition();
    int adjust_date();
    int open_gate();
    int close_gate();
    int reset_pidc();
    int init_pidc();
    int init_excl();

    int m_moduleNo;
    int m_in_status;
    int m_out_status;
    std::string m_pulIdInitVal;
    std::string m_exTrgInitVal;
    std::string m_gatenetAddr;

    bool m_usePulseInitVal;
    bool m_useExTrgInitVal;

    ///static const std::string GATENET_ADDR;    ///"192.168.0.15";
    static const std::string REGISTER_GATE;   ///"gate";
    static const std::string REGISTER_DATE;   ///"current_time";
    static const std::string REGISTER_PIDC;   ///"pulse_id_cnt";
    static const std::string REGISTER_EXCL;   ///"extclock";
    static const std::string VAL_GATE_OPEN;   ///"0x00800080"
    static const std::string VAL_GATE_CLOSE;  ///"0x00800000"
    static const std::string VAL_PIDC_RESET;  ///"0x0000000000"
    static const std::string CONDITION_FILE;  ///"./condition.json"
    static const int MAX_NUM_VALUE =  16;
    static const int MAX_BUF_BYTE  = 256;

    bool m_debug;
};


extern "C"
{
    void GatenetInit(RTC::Manager* manager);
};

/// pmctl is used for writing/reading to/from Gatenet registers via UDP.
extern "C"
{
#include "pm_reg.h"
};

#endif // SKELETON_H
