// -*- C++ -*-
/*!
 * @file Gatherer.h
 * @brief Event data gathering component from PSD-SiTCP(NEUNET) modules
 * @date 
 * @author Kazuo Nakayoshi <kazuo.nakayoshi@kek.jp>
 *
 * Copyright (C) 2008
 *     Kazuo Nakayoshi
 *     Electronics System Group,
 *     KEK, Japan.
 *     All rights reserved.
 *
 */

#ifndef GATHERER_H
#define GATHERER_H

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <rtm/idl/BasicDataTypeSkel.h>

#include "MyRingBuffer.h"
#include "MlfComponent.h"
#include "ComponentInfoContainer.h"
#include "Neunet.h"
#include "Selector.h"
#include "ConditionGatherer.h"

using namespace RTC;

typedef std::vector< std::string > stringList;

class Gatherer
    : public DAQMW::MlfComponent
{
public:
    Gatherer(RTC::Manager* manager);
    ~Gatherer();

    // The initialize action (on CREATED->ALIVE transition)
    // formaer rtc_init_entry() 
    virtual RTC::ReturnCode_t onInitialize();

    // The execution action that is invoked periodically
    // former rtc_active_do()
    virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

private:
    TimedOctetSeq m_out_data;
    OutPort<TimedOctetSeq, MyRingBuffer> m_OutPort;

private:
    int daq_dummy();
    int daq_configure();
    int daq_unconfigure();
    int daq_start();
    int daq_run();
    int daq_stop();
    int daq_pause();
    int daq_resume();
    
    int check_outPort_status(int status);
    int set_data_to_OutPortBuf(RTC::TimedOctetSeq* out_data, unsigned char* buf, 
			       unsigned addr, unsigned daqId, unsigned modNo, 
			       unsigned event_num, unsigned seq_num);
  ///int connect_modules(DAQMW::Selector& m_selector, stringList myList);
    int connect_modules();
    int disconnect_modules();

    int parse_params(::NVList* list);
    int set_condition(); 
    int set_params(ConditionGatherer* condGatherer);
    int set_lld_tof(std::string addr, neunetParams* neuParams);
  /*
    int get_neunet_num(stringList* dataSrcAddr);
    int get_gatenet_modNo(stringList* dataSrcAddr);
    int get_t0_sampling_modNo(stringList* dataSrcAddr);
  */
    DAQMW::Selector m_selector;
    DAQMW::psd_module_list*  m_readyList;

    stringList m_dataSrcAddr;
    unsigned int  m_packetSeqNum;
    int m_out_status;
    int m_select_tout_counter;
    int m_loop_tm;
    unsigned m_t0_sampling_modNo;
    long long m_t0_counter;
    long long m_t0_max;

    struct timeval m_start_tm;
    struct timeval m_stop_tm;

    bool m_requested;
    bool m_condStopMode;
    bool m_finished;
    bool m_debug;

    static const int REQUEST_EVENT_NUM = 4*1024;
    static const int NEUNET_BUF_BYTE_SIZE = 64*1024;
    static const int SELECT_TIMEOUT = 2;
    static const unsigned char NEUNET_EVENT_MAGIC = 0x5a;
    static const unsigned char NEUNET_T0_MAGIC    = 0x5b;
    static const std::string GATENET_ADDR;
    static const std::string CONDITION_T0_CNT;
    static const std::string CONDITION_FILE;
};

extern "C"
{
    void GathererInit(RTC::Manager* manager);
};

/// pmctl is used for writing/reading to/from Gatenet registers via UDP.
extern "C"
{
#include "pm_reg.h"
};

#endif // GATHERER_H
