// -*- C++ -*-
/*!
 * @file Logger.h
 * @brief Event data logging component.
 * @date
 * @author Kazuo Nakayoshi <kazuo.nakayoshi@kek.jp>
 *
 * Copyright (C) 2008
 *     Kazuo Nakayoshi
 *     Electronics System Group,
 *     KEK, Japan.
 *     All rights reserved.
 *
 */

#ifndef LOGGER_H
#define LOGGER_H

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <rtm/idl/BasicDataTypeSkel.h>
#include "MlfComponent.h"
#include <rtm/RingBuffer.h>
#include "MyRingBuffer.h"
#include "FileUtils.h"

// Service implementation headers
#include "DAQServiceSVC_impl.h"

using namespace RTC;

class Logger
    : public DAQMW::MlfComponent
{
public:
    Logger(RTC::Manager* manager);
    ~Logger();

    // The initialize action (on CREATED->ALIVE transition)
    virtual RTC::ReturnCode_t onInitialize();

    // The execution action that is invoked periodically
    virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

private:
    TimedOctetSeq m_in_data;
    InPort<TimedOctetSeq, MyRingBuffer> m_InPort;

private:
    int daq_dummy();
    int daq_configure();
    int daq_unconfigure();
    int daq_start();
    int daq_run();
    int daq_stop();
    int daq_pause();
    int daq_resume();

    int parse_params(::NVList* list);
    int reset_InPort();

    FileUtils* fileUtils;
    bool m_isDataLogging;
    bool m_filesOpened;
    std::string m_dirName;
    std::string m_instId;
    std::string m_daqId_str;
  
    unsigned int m_maxFileSizeInMByte;
    std::vector< std::string >  m_moduleNameList;
    std::vector<std::ofstream*>  m_file_list;
    int m_in_status;
    int m_update_rate;
    bool m_debug;
};

extern "C"
{
    void LoggerInit(RTC::Manager* manager);
};

#endif // LOGGER_H
