// -*- C++ -*-
/*!
 * @file Monitor.h
 * @brief Event data monitoring component.
 * @date
 * @author Kazuo Nakayoshi <kazuo.nakayoshi@kek.jp>
 *
 * Copyright (C) 2008
 *     Kazuo Nakayoshi
 *     Electronics System Group,
 *     KEK, Japan.
 *     All rights reserved.
 *
 */

#ifndef MONITOR_H
#define MONITOR_H

#include <bitset>
#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <rtm/idl/BasicDataTypeSkel.h>
#include <rtm/RingBuffer.h>
#include "MlfComponent.h"
#include "DAQServiceSVC_impl.h"
#include "MyRingBuffer.h"

////////// Manyo Monitor //////////
#include "EventDataToHistogramEmulator.h"
#include "GslHistogram.hh"
#include "GnuplotInterface.hh"
#include "StringTools.hh"
#include "GslHistogram.hh"
#include "OnLineGnuPlot.hh"
//////// Manyo Monitor End ////////

//////////// Condition ////////////
#include "ConditionMonitor.h"
////////// Condition End //////////

using namespace RTC;

class Monitor
   : public DAQMW::MlfComponent
{
public:
    Monitor(RTC::Manager* manager);
    ~Monitor();
    virtual RTC::ReturnCode_t onInitialize();
    virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

private:
    int daq_dummy();
    int daq_configure();
    int daq_unconfigure();
    int daq_start();
    int daq_run();
    int daq_stop();
    int daq_pause();
    int daq_resume();
    int parse_params(::NVList* list);
    int reset_InPort();
	int check_gatherer_header_footer(int comp_seq_num, unsigned char *header,
									 unsigned char *footer);

private:
	// Constants
    static const int EVENT_BYTE_SIZE = 8;
    static const int MAX_EVENT_NUM   = 4096;
    static const int MAX_BUF_SIZE    = MAX_EVENT_NUM * EVENT_BYTE_SIZE;

	// RTC //
    TimedOctetSeq m_in_data;
    InPort<TimedOctetSeq, MyRingBuffer> m_InPort;
    unsigned char m_cdata[MAX_BUF_SIZE];
    int m_in_status;
    int m_sampling_rate;
    int m_update_rate;

	// Generic //
    bool  m_debug;
    bool  m_monitor_debug;
    bool  m_monitor_debug_verbose;

	// Condition //
	monitorParam m_monitorParam;
	int set_condition();

	// Manyo Monitor //
	//
    UInt4  m_num_of_psd_per_module;
    UInt4  m_total_module_num;
    UInt4  m_daqId;
	//
    std::string m_gnuplot_path;
    std::string m_png_output_dir;
	//
    GnuplotInterface             *m_gplot;
    StringTools                  *m_stool;
    EventDataToHistogramEmulator *m_evs;
    OnLineGnuPlot                *m_OnLineGp;
    vector<GslHistogram*>        *m_vg;
	//
	int print_monitor_param(void);
	int create_png_files(void);
    int create_tof_files(void);
    int create_1d_position_files(void);
    int create_2d_position_file(void);

// If we don't use Manyo, we have to implement event data decode routine.
//
// PSD Event Data
// struct psd_event {
//     unsigned char sig:  8;
//     unsigned int  tof: 24;
//     unsigned char id:   8;
//     unsigned int phLR: 24;
// };
// typedef struct psd_event psdEvent;
//
// Decode Routine
// bool decode(
//     unsigned char*  event,
//     unsigned int*   module_id, 
//     unsigned int*   psd_id, 
//	   unsigned int*   tof,
//     unsigned short* phl,
//     unsigned short* phr, 
//	   unsigned int*   pha,
//     double*         pos
// );

};

extern "C" {
  void MonitorInit(RTC::Manager* manager);

};
#endif // MONITOR_H
