// -*- C++ -*-
/*!
 * @file MonitorComp.cpp
 */

#include <rtm/Manager.h>
#include <iostream>
#include "Monitor.h"
#include <rtm/NVUtil.h>

void MyModuleInit(RTC::Manager* manager)
{
  MonitorInit(manager);

  RtcBase* comp;
  std::cout << "Creating a component: \"Monitor\"....";
  comp = manager->createComponent("Monitor");
  std::cout << "succeed." << std::endl;

  RTC::ComponentProfile_var prof;
  prof = comp->get_component_profile();
  std::cout << "=================================================" << std::endl;
  std::cout << " Component Profile" << std::endl;
  std::cout << "-------------------------------------------------" << std::endl;
  std::cout << "InstanceID:     " << prof->instance_name << std::endl;
  std::cout << "Implementation: " << prof->type_name << std::endl;
  std::cout << "Description:    " << prof->description << std::endl;
  std::cout << "Version:        " << prof->version << std::endl;
  std::cout << "Maker:          " << prof->vendor << std::endl;
  std::cout << "Category:       " << prof->category << std::endl;
  std::cout << "  Other properties   " << std::endl;
  NVUtil::dump(prof->properties);
  std::cout << "=================================================" << std::endl;

  PortList* portlist;
  portlist = comp->get_ports();

  for (CORBA::ULong i(0), n(portlist->length()); i < n; ++i)
    {
      Port_ptr port;
      port = (*portlist)[i];
      std::cout << "================================================="
                << std::endl;
      std::cout << "Port" << i << " (name): ";
      std::cout << port->get_port_profile()->name << std::endl;
      std::cout << "-------------------------------------------------"
                << std::endl;
      RTC::PortInterfaceProfileList iflist;
      iflist = port->get_port_profile()->interfaces;

      for (CORBA::ULong i(0), n(iflist.length()); i < n; ++i)
        {
          std::cout << "I/F name: ";
          std::cout << iflist[i].instance_name << std::endl;
          std::cout << "I/F type: ";
          std::cout << iflist[i].type_name << std::endl;
          const char* pol;
          pol = iflist[i].polarity == 0 ? "PROVIDED" : "REQUIRED";
          std::cout << "Polarity: " << pol << std::endl;
        }
      std::cout << "- properties -" << std::endl;
      NVUtil::dump(port->get_port_profile()->properties);
      std::cout << "-------------------------------------------------" << std::endl;
    }

  RTC::RTObject_var rtobj;
  rtobj = RTC::RTObject::_narrow(manager->getPOA()->servant_to_reference(comp));

  ExecutionContextServiceList_var eclist;
  eclist = rtobj->get_execution_context_services();
  eclist[0]->activate_component(RTObject::_duplicate( rtobj ));

  return;
}


int main (int argc, char** argv)
{
   //TROOT root("GUI","GUI");

  RTC::Manager* manager;
  manager = RTC::Manager::init(argc, argv);
  //TApplication theApp("App", &argc, argv);

  // Initialize manager
  manager->init(argc, argv);

  // Set module initialization proceduer
  // This procedure will be invoked in activateManager() function.
  manager->setModuleInitProc(MyModuleInit);

  // Activate manager and register to naming service
  manager->activateManager();

  // run the manager in blocking mode
  // runManager(false) is the default.
  manager->runManager();

  //theApp.Run();
  // If you want to run the manager in non-blocking mode, do like this
  // manager->runManager(true);

  return 0;
}

