#include "Monitor.h"

extern std::string cformat(char *format, ...);

std::string get_output_filepath(string output_dir, UInt4 daqId, UInt4 mod_num, UInt4 psd_num)
{
	Char buf[1024];
	sprintf(buf, "set output \'%s/pos_%d_%d_%d.png\'",
		output_dir.c_str(), daqId, mod_num, psd_num);
	string s = string(buf);
	return s;
}

int Monitor::create_png_files(void)
{
    create_2d_position_file();
    create_tof_files();
    create_1d_position_files();

	return 0;
}

int Monitor::create_1d_position_files(void)
{
	// (m_loop / m_monitir_update_rate) = 1, 2, 3, ... (start is 1, not 0)
	// We write 1D postion png file of only one module.

	UInt4 pixel_id;

	UInt4 num_of_psd_module = m_total_module_num;
	UInt4 modNo = (m_loop / m_monitorParam.monitor_update_rate) % num_of_psd_module;

	if (m_monitor_debug) {
		std::cerr << "===> Writing 1D Pos: module # " << modNo << std::endl;
	}

	for(UInt4 psd_num = 0; psd_num <m_num_of_psd_per_module; psd_num ++) {

		// m_gplot->e("set term png");
		// m_gplot->e(get_output_filepath(m_png_output_dir, m_daqId, modNo, psd_num));
		// m_gplot->e("plot [0:" + m_stool->Int4ToString(m_num_of_pixel_per_psd) +
		// 				"] '-' with lp");
		// for (UInt4 pixel = 0; pixel < m_num_of_pixel_per_psd; pixel ++) {
		// 	pixel_id = modNo*m_num_of_psd_per_module*m_num_of_pixel_per_psd +
		// 			   psd_num*m_num_of_pixel_per_psd +
		// 			   pixel;
		// 	m_gplot->e(m_stool->Int4ToString(pixel) + " " +
		// 			   m_stool->DoubleToString(m_evs->PutSum(pixel_id)));
		// }
		// m_gplot->e("e");

		m_gplot->e(cformat("set term png"));
		m_gplot->e(cformat("set output '%s/pos_%d_%d_%d.png'", m_png_output_dir.c_str(), m_daqId, modNo, psd_num));
		m_gplot->e(cformat("plot [%d:%d] '-' with lp", 0, m_monitorParam.num_of_pixel_per_psd));

		for (UInt4 pixel = 0; pixel < m_monitorParam.num_of_pixel_per_psd; pixel ++) {
			pixel_id = modNo*m_num_of_psd_per_module*m_monitorParam.num_of_pixel_per_psd +
					   psd_num*m_monitorParam.num_of_pixel_per_psd +
					   pixel;
			m_gplot->e(cformat("%d %f", pixel, m_evs->PutSum(pixel_id)));
		}

		m_gplot->e("e");
	}

	return 0;
}

int Monitor::create_tof_files()
{
	// (m_loop / m_monitir_update_rate) = 1, 2, 3, ... (start is 1, not 0)
	// We write TOF png file of only one module.

    UInt4 num_of_psd_module = m_total_module_num;
    UInt4 psd_module = (m_loop / m_monitorParam.monitor_update_rate) % num_of_psd_module;
    //std::cerr << "===> Writing TOF: 192.168.0." << psd_module + 16 <<
	if (m_monitor_debug) {
    	std::cerr << "===> Writing TOF: module # " << psd_module << std::endl;
	}

    for(UInt4 i = 0; i < m_num_of_psd_per_module; i ++) {
        UInt4 psd_num = psd_module*m_num_of_psd_per_module + i; 
		if (m_monitor_debug_verbose) {
        	std::cerr << "TOF --> psd_num: " << psd_num <<std::endl;
		}

        // m_OnLineGp->e("set terminal png");
        // m_OnLineGp->e("set output '" + m_png_output_dir + "/tof_" +
		// 				m_stool->Int4ToString(m_daqId) +
		// 				"_" +
        //    			m_stool->Int4ToString(psd_module) +
        //    			"_" +
        //    			m_stool->Int4ToString(i) +
        //    			".png'"
        //);
        //m_OnLineGp->e("set xlabel 'u sec'");
        //m_OnLineGp->e("set title 'PSD # " + m_stool->Int4ToString(psd_num) +
		//				"'"
		//);

        m_OnLineGp->e(cformat("set terminal png"));
        m_OnLineGp->e(cformat("set output '%s/tof_%d_%d_%d.png'",
			m_png_output_dir.c_str(),
			m_daqId, psd_module,
			i));
        m_OnLineGp->e(cformat("set xlabel 'u sec'"));
        m_OnLineGp->e(cformat("set title 'PSD # %d'", psd_num));
        m_OnLineGp->TofVsCounts(psd_num);
    }

    return 0;
}

int Monitor::create_2d_position_file()
{
	if (m_monitor_debug) {
		std::cerr << "===> Writing 2D Position" << std::endl;
	}

	//string filename = "/dev/shm/pos_2d_" 
	//	+ m_stool->Int4ToString(m_daqId)
	//	+ ".dat";

	string filename = cformat("/dev/shm/pos_2d_%d.dat", m_daqId);

	m_OnLineGp->Create2DHistogramDataFile(filename);

	//m_OnLineGp->e("set terminal png");
	//m_OnLineGp->e("set output '" + m_png_output_dir + "/pos_2d_" +
	//				 m_stool->Int4ToString(m_daqId) + ".png'");
    //m_OnLineGp->e("set pm3d map");
    //m_OnLineGp->e("set xlabel 'Position (relative)'");
    //m_OnLineGp->e("set title 'Position Crate # X'");
    //m_OnLineGp->e("set title 'Position DAQ ID # " +
	//				m_stool->Int4ToString(m_daqId) + "'");
    //m_OnLineGp->e("splot '" + filename + "'");

	m_OnLineGp->e(cformat("set terminal png"));
	m_OnLineGp->e(cformat("set output '%s/pos_2d_%d.png'", m_png_output_dir.c_str(), m_daqId));
    m_OnLineGp->e(cformat("set pm3d map"));
    m_OnLineGp->e(cformat("set xlabel 'Position (relative)'"));
    m_OnLineGp->e(cformat("set title 'Position DAQ ID # %d'", m_daqId));
    m_OnLineGp->e(cformat("splot '%s'", filename.c_str()));

	if (m_monitor_debug) {
    	std::cerr << "===> Writing 2D Position End" << std::endl;
	}
    
    return 0;
}
