// -*- C++ -*-
/*!
 * @file RingBuffer.h
 * @brief Defautl Buffer class
 * @date $Date$
 * @author Noriaki Ando <n-ando@aist.go.jp>
 * modified : 2007/06/14 for block read/write
 *
 * Copyright (C) 2006
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id$
 *
 */

/*
 * $Log: RingBuffer.h,v $
 * Revision 1.5  2007/04/26 15:32:56  n-ando
 * The header include order was modified to define _REENTRANT before
 * including ace/config-lite.h in Linux systems.
 * In ace 5.4.7 or later, _REENTRANT flag should be defined explicitly.
 *
 * Revision 1.4  2007/01/06 18:02:49  n-ando
 * The RingBuffer has been completely re-implemented.
 *
 *
 */

#ifndef MyRingBuffer_h
#define MyRingBuffer_h

#include <rtm/RTC.h>
#include <sys/time.h>
#include <pthread.h>
#include <asm/errno.h>
#include <algorithm>
//#include <ace/Synch.h>
#include <rtm/BufferBase.h>
//#define TIMEOUT_TIME 100000000 // 100 msec in nano
#define TIMEOUT_TIME_MILLI 100 // 100 msec in milli

  template <class DataType>
  class MyRingBuffer
    : public RTC::BufferBase<DataType>
  {
  public:
    MyRingBuffer(long int length)
      : m_Length(length < 2 ? 2 : length),
	m_WritePointer(0), m_ReadPointer(0), 
	m_Full(0)
      //	 m_Length(length < 2 ? 2 : length - 1)
    {
      m_Buffer.resize(m_Length);
      std::cout << "MyRingBuffer::m_Length = " << m_Length << std::endl;
      pthread_mutex_init(&m_mut, 0);
      pthread_cond_init(&m_cond_Full, 0);
      pthread_cond_init(&m_cond_Empty, 0);
    }

    /*!
     * @if jp
     *
     * @brief ۥǥȥ饯
     * @else     *
     * @brief virtual destractor
     *
     * @endif
     */
    virtual ~MyRingBuffer(){};
    
    /*
    void init(DataType& data)
    {
      for (long int i = 0; i < m_length; ++i)
	{
	  put(data);
	}
    }
    */
    /*!
     * @if jp
     *
     * @brief ХåեĹ
     * 
     * @else
     *
     * @brief Get the buffer length
     *
     * @endif
     */
    virtual long int length() const
    {
      return m_Length;
    }

    /*!
     * @if jp
     *
     * @brief Хåե˥ǡ񤭹
     * 
     * @else
     *
     * @brief Write data into the buffer
     *
     * @endif
     */

    virtual int write(const DataType& value)
    {
      return put(value);
    }

    /*!
     * @if jp
     *
     * @brief Хåե˥ǡ񤭹
     * 
     * @else
     *
     * @brief Write data into the buffer
     *
     * @endif
     */
    virtual int read(DataType& value)
    {
      return get(value);
    }

    /*!
     * @if jp
     *
     * @brief ХåեfullǤ
     * 
     * @else
     *
     * @brief True if the buffer is full, else false.
     *
     * @endif
     */
    virtual bool isFull() const
    {
      return false;
    }
    /*!
     * @if jp
     *
     * @brief ХåեemptyǤ
     * 
     * @else
     *
     * @brief True if the buffer is empty, else false.
     *
     * @endif
     */
    virtual bool isEmpty() const
    {
      return !(this->isNew());
    }

    bool isNew() const
    {
      return m_Buffer[m_ReadPointer].isNew();
    }

  protected:

    /*!
     * @if jp
     *
     * @brief Хåե˥ǡ񤭹
     * 
     * @else
     *
     * @brief Write data into the buffer
     *
     * @endif
     */

    pthread_mutex_t m_mut;
    pthread_cond_t m_cond_Full;
    pthread_cond_t m_cond_Empty;

    struct timeval now_put;
    struct timezone tz_put;
    struct timespec timeout_put;

    inline int put(const DataType& value)
    {
      int status = BUF_SUCCESS;
      //      std::cout << "MyRingBuffer put() method..." << std::endl;
      pthread_mutex_lock(&m_mut);
      gettimeofday(&now_put, &tz_put);

      ///timeout_put.tv_sec = now_put.tv_sec + 1; // time out time is 1 sec.
      ///timeout_put.tv_nsec = now_put.tv_usec * 1000;

      timeout_put.tv_sec  =  now_put.tv_sec  +  TIMEOUT_TIME_MILLI / 1000;
      timeout_put.tv_nsec = (now_put.tv_usec + (TIMEOUT_TIME_MILLI % 1000)*1000) * 1000;
      
      if (timeout_put.tv_nsec >= 1000000000) {
	timeout_put.tv_sec++;
	timeout_put.tv_nsec -= 1000000000;
      }


      if(m_Full == (m_Length-1)) {
	 //std::cout << "MyRingBuffer:Buffer full..." << std::endl;
	while(m_Full == (m_Length-1)) {
	  status = pthread_cond_timedwait(&m_cond_Full, &m_mut, &timeout_put);
	 if(status == ETIMEDOUT)
	   status = BUF_TIMEOUT;
	 else 
	   if(status == EINTR)
	     status = BUF_FATAL;
	 if(status) // not success, break
	   break; // 
	 else
	   status = BUF_SUCCESS;
	}
      }
      if(status == BUF_SUCCESS) {
	m_Buffer[m_WritePointer].write(value);
	m_WritePointer = (++m_WritePointer) % m_Length;
	++m_Full;
	pthread_cond_broadcast(&m_cond_Empty);
      }
      pthread_mutex_unlock(&m_mut);
 
     return status;
    }

    /*!
     * @if jp
     *
     * @brief Хåեǡ
     * 
     * @else
     *
     * @brief Get data from the buffer
     *
     * @endif
     */
    struct timeval now_get;
    struct timezone tz_get;
    struct timespec timeout_get;

    virtual int get(DataType& value)
    {
      int status = BUF_SUCCESS;
      pthread_mutex_lock(&m_mut);
      gettimeofday(&now_get, &tz_get);

      //timeout_get.tv_sec = now_get.tv_sec + 1; // time out time is 1 sec.
      //timeout_put.tv_nsec = now_put.tv_usec * 1000;

      timeout_get.tv_sec  =  now_get.tv_sec  +  TIMEOUT_TIME_MILLI / 1000;
      timeout_get.tv_nsec = (now_get.tv_usec + (TIMEOUT_TIME_MILLI % 1000)*1000) * 1000;
      
      if (timeout_get.tv_nsec >= 1000000000) {
	timeout_get.tv_sec++;
	timeout_get.tv_nsec -= 1000000000;
      }

      if(m_Full == 0) {
	 //std::cout << "MyRingBuffer:Buffer empty..." << std::endl;
	while(m_Full == 0) {
	  status = pthread_cond_timedwait(&m_cond_Empty, &m_mut, &timeout_get);
	 if(status == ETIMEDOUT)
	   status = BUF_TIMEOUT;
	 else 
	   if(status == EINTR)
	     status = BUF_FATAL;
	 if(status) // not success, break
	   break; // 
	 else
	   status = BUF_SUCCESS;
	}
      }
      if(status == BUF_SUCCESS) {
	value = m_Buffer[m_ReadPointer].read();
	m_ReadPointer = (++m_ReadPointer) % m_Length;
	--m_Full;
	pthread_cond_broadcast(&m_cond_Full);
      }
      pthread_mutex_unlock(&m_mut);

      return status;
    }

    virtual DataType& getRef()
    {
      int readPointer = (--m_ReadPointer) % m_Length;
      return m_Buffer[readPointer].data;
    }

  private:
    long int m_Length;
    int m_WritePointer;
    int m_ReadPointer;
    int m_Full;

    /*!
     * @if jp
     * @brief Хåե
     * @else
     * @brief Buffer sequence
     * @endif
     */
    template <class D>
    class Data
    {
    public:
      Data() : data(), is_new(false){;}
      inline Data& operator=(const D& other)
      {
        this->data = other;
        this->is_new = true;
        return *this;
      }
      inline void write(const D& other)
      {
        this->is_new = true;
        this->data = other;
      }
      inline D& read()
      {
        this->is_new = false;
        return this->data;
      }
      inline bool isNew() const
      {
        return is_new;
      }
      D    data;
      bool is_new;
    };

    std::vector<Data<DataType> > m_Buffer;

  };

#endif // MyRingBuffer_h
