#include "Neunet.h"

namespace DAQMW {
    Neunet::Neunet()
    {
      std::cerr << "Neunet created\n";
    }

    Neunet::~Neunet()
    {
      std::cerr << "Neunet deleted\n";
    }

    int Neunet::init(const std::string host, int mod_num)
    {
	initTcp(host);
	setModuleNumber(mod_num);
	return 0;
    }

    int Neunet::init(const std::string host, int buf_size, int mod_num)
    {
	initTcp(host);
	setBufSize(buf_size);
	setModuleNumber(mod_num);
	return 0;
    }

    int Neunet::initTcp(std::string host)
    {
	return PsdModule::initTcp(host, NEUNET_PORT_TCP);
    }

    int Neunet::requestEvent(unsigned int event_num )
    {
	int status;
	unsigned char packet[sizeof(psd_request)];

	setCmdEventLen(event_num);
	pack_tcp((unsigned int*)packet);
	status = write(packet, (int)sizeof(psd_request));
	
	if ((status == Sock::ERROR_FATAL)||(status == Sock::ERROR_TIMEOUT)) {
	    return status;
	}
	return 0;
    }

    int Neunet::readEventNum()
    {
	int ret = 0;
	int word_len = 0;

	ret = readAll( EVENT_LEN_SIZE );
	if (ret == Sock::ERROR_FATAL) {
	    return ret;
	}
	else {
	    unsigned char* buffer = getEvent();
	    word_len = *(int*)buffer; 
	}

	return ( wordToEventNum( ntohl(word_len) ) );
    }

    unsigned char* Neunet::readEvent(int event_num)
    {
	if (readAll(event_num * EVENT_BYTE_SIZE) == Sock::ERROR_FATAL) {
	    return 0;
	}
	unsigned char* buffer = getEvent();
	return buffer;
    }

    int Neunet::eventNumToWord(int event_num)
    {
	return (event_num * EVENT_BYTE_SIZE / NEUNET_WORD_SIZE);
    }

    int Neunet::wordToEventNum(int word_len)
    {
	return ( word_len * NEUNET_WORD_SIZE / EVENT_BYTE_SIZE );
    }

    int Neunet::setCmdEventLen(int event_num)
    {
	m_request.len.length = Neunet::eventNumToWord(event_num);
	m_request.len.cmd = NEUNET_CMD_READ_LEN;
	return 0;
    }

    int Neunet::pack_tcp(unsigned int* packet) {

	packet[0] = (int)m_request.len.cmd;
	packet[1] = htonl(m_request.len.length);
	return 0;
    }

    int Neunet::unpack_tcp(unsigned int* packet, psd_response* response) 
    {
	unsigned int c[2];

	c[0] = ntohl(packet[1]);
	c[1] = ntohl(packet[0]);
	memcpy(response, c, sizeof(psd_response));
	return 0;
    }

    bool Neunet::isEventData(psd_response* response) {
	bool ret;
	if(response->event.cmd == NEUNET_CMD_EVENT_DATA) {
	    ret = true;
	}
	else {
	    ret = false;
	}
	return ret;
    }

    int Neunet::extract_event_data(psd_response* event_data,
				       unsigned int* tof,
				       unsigned char* psd,
				       unsigned short* pulse_left,
				       unsigned short* pulse_right) {
	unsigned int pulse;
	unsigned int t;

	std::cerr << "psd_extract_event_data: cmd = %x\n"
		  << event_data->event.cmd << std::endl;

	t = 0;
	memcpy((char*)&t, (char*)&event_data->event.t0[0], 3);
	*tof = t;
	*psd = event_data->event.psd;
	memcpy((char*)&pulse, (char*)&event_data->event.pulse[0], 4);
	*pulse_right = pulse & 0x0FFF;
	*pulse_left  = (pulse>>12) & 0x0FFF;

	return 0;
    }

    int Neunet::getModuleNumber() {
	return m_moduleNumber;
    }

    int Neunet::setModuleNumber( int modNum ) {
	m_moduleNumber = modNum;
	return 0;
    }
    
} //namespace DAQMW 
