// -*- C++ -*-
#ifndef NEUNET_H
#define NEUNET_H

#include "PsdModule.h"

namespace DAQMW {

    struct read_len {
	unsigned int length;
	unsigned char dummy[3];
	unsigned char cmd;
    };

    struct mem_ctrl {
	unsigned char dummy[4];
	unsigned char dl;
	unsigned char dh;
	unsigned char ah;
	unsigned char cmd;
    };
    
    typedef union psd_request {
	struct read_len len;
	struct mem_ctrl ctl;
	unsigned char data[8];
    } psd_request;
  
    struct event_data {
	unsigned char pulse[3];
	unsigned char psd;
	unsigned char t0[3];
	unsigned char cmd;
    };

    struct t0_data {
	unsigned char k[5];
	unsigned char m;
	unsigned char c;
	unsigned char cmd;
    };

    typedef union psd_response {
	struct event_data event;
	struct t0_data t0;
	unsigned char data[8];
    } psd_response;

    class Neunet:
	public DAQMW::PsdModule 
    {

    public:
	Neunet();
	virtual ~Neunet();

	int init(const std::string host, int mod_num);
	int init(const std::string host, int buf_size, int mod_num);
	int initTcp(const std::string host);
	int requestEvent(unsigned int event_num = MAX_EVENT_NUM);
	int readEventNum();
	unsigned char* readEvent(int event_num);
	bool isEventData(psd_response* response);
	int extract_event_data(psd_response* event_data,
				   unsigned int* time,
				   unsigned char* psd,
				   unsigned short* pulse_left,
				   unsigned short* pulse_right);
	int unpack_tcp(unsigned int* packet, psd_response* response);
	int getModuleNumber();
	int setModuleNumber(int modNum);

	static const unsigned int EVENT_BYTE_SIZE  = 8;
	static const unsigned int NEUNET_WORD_SIZE = 2;
	static const unsigned int EVENT_LEN_SIZE   = 4; //4bye
	static const unsigned int MAX_EVENT_NUM    = 4 * 1024; //4k events

    private:
	int m_moduleNumber;

	static const unsigned NEUNET_PORT_TCP       = 23;
	static const unsigned NEUNET_PORT_UDP       = 1234;
	
	static const unsigned NEUNET_WRITE_LEN      = 32792; // in bytes
    
	static const unsigned NEUNET_CMD_READ_LEN   = 0xa3;
	static const unsigned NEUNET_CMD_MEM_CTL    = 0xa2;
	static const unsigned NEUNET_CMD_EVENT_DATA = 0x5a;
	static const unsigned NEUNET_CMD_T0_DATA    = 0x5b;

	// NEUNET regisgers
	static const unsigned NEUNET_ADDR_MACADR    = 0x40;
	static const unsigned NEUNET_SIZE_MACADR    = 6; // in bytes
	static const unsigned NEUNET_ADDR_COMMENT   = 0x52;
	static const unsigned NEUNET_SIZE_COMMENT   = 38; // in bytes
	static const unsigned NEUNET_ADDR_USED_HOUR = 0x9C;
	static const unsigned NEUNET_SIZE_USED_HOUR = 4; // in bytes
	static const unsigned NEUNET_ADDR_PULSE_ID_CNT = 0x18B;
	static const unsigned NEUNET_SIZE_PULSE_ID_CNT = 5; // in bytes
	static const unsigned NEUNET_ADDR_TIMER     = 0x190;
	static const unsigned NEUNET_SIZE_TIMER     = 4; // in bytes
	static const unsigned NEUNET_ADDR_LLD_TIME  = 0x198;
	static const unsigned NEUNET_SIZE_LLD_TIME  = 8; // in bytes


	int eventNumToWord(int event_num);
	int wordToEventNum(int word_num);
	int setCmdEventLen(int event_num);
	int pack_tcp(unsigned int* packet);

	psd_request  m_request;
	psd_response m_response;

    };

}

#endif
