// -*- C++ -*-

#ifndef MODULE_H
#define MODULE_H

#include <vector>
#include <iostream>
#include <sstream>
#include "Sock.h"

namespace DAQMW {
    
    class PsdModule
    {
    public: 
	PsdModule()
	    : m_socketTcp(0), m_buffer(0), m_buf_byte_size(0), m_debug(false)
	{ 
	    std::cerr << "Module created\n";
	}

	virtual ~PsdModule()
	{
	    if(m_buffer) {
		delete [] m_buffer;
	    }
	    std::cerr << "Module deleted\n";
	}

	PsdModule(const PsdModule& obj)
	{
	    std::cerr << "Module copy constructor" << std::endl;
	    Sock* m_socketTcp = new Sock();
	    *m_socketTcp = *obj.m_socketTcp;

	    m_buffer = new unsigned char[m_buf_byte_size];
	    for (int i = 0; i < m_buf_byte_size; i++) {
		m_buffer[i] = obj.m_buffer[i];
	    }
	}


	int initTcp(const std::string host, const int port)
	{ 
	    m_socketTcp = new Sock(host, port);

	    std::istringstream is(host);
	    int octet1, octet2, octet3, octet4;
	    char dot1, dot2, dot3;
	    is >> octet1 >> dot1 >> octet2 >> dot2 >> octet3 >> dot3 >> octet4;
	    if (m_debug) {
		std::cerr << "octet4=" << octet4 << std::endl;
	    }
	    m_ipAddress.push_back(octet1);
	    m_ipAddress.push_back(octet2);
	    m_ipAddress.push_back(octet3);
	    m_ipAddress.push_back(octet4);

	    return 0;
	}

	void setBufSize(int buf_byte_size)
	{
	    std::cerr << "setBufSize:" << buf_byte_size << std::endl;
	    m_buffer = new unsigned char[buf_byte_size];
	    m_buf_byte_size = buf_byte_size;
	}

	int setBuffer(unsigned char* buffer)
	{
	    m_buffer = buffer;
	    return 0;
	}

	int connectTcp() 
	{
	    return m_socketTcp->connectTCP();
	}

	void disconnectTcp()
	{
	    return m_socketTcp->disconnect();
	}

	int read(int nbytes)
	{
	    int n = m_socketTcp->read(m_buffer, nbytes);
	    std::cerr << "read n=" << n << std::endl;
	    for (int i=0; i< n; i++) {
		std::cerr << (int)m_buffer[i];
	    }

	    return n;
	}

	int readAll(int nbytes)
	{
	    int n = m_socketTcp->readAll(m_buffer, nbytes);
	    return n;
	}

	int write(unsigned char* buffer, int nbytes)
	{
	    int n = m_socketTcp->write(buffer, nbytes);
	    return n;
	}

	unsigned char* getEvent()
	{
	    return m_buffer;
	}

	int getSockFd()
	{
	    return m_socketTcp->getSockFd();
	}

	std::vector<unsigned char>* getIpAddress()
	{
	    return &m_ipAddress;
	}

    private:
	Sock* m_socketTcp;
	unsigned char* m_buffer;
	int m_buf_byte_size;
	std::vector<unsigned char> m_ipAddress;
	bool m_debug;
    };
}
#endif
